package com.planet_ink.coffee_mud.Items.interfaces;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;
import java.util.Vector;

import com.planet_ink.coffee_mud.core.interfaces.*;
import com.planet_ink.coffee_mud.core.*;
import com.planet_ink.coffee_mud.core.collections.*;
import com.planet_ink.coffee_mud.Abilities.interfaces.*;
import com.planet_ink.coffee_mud.Areas.interfaces.*;
import com.planet_ink.coffee_mud.Behaviors.interfaces.*;
import com.planet_ink.coffee_mud.CharClasses.interfaces.*;
import com.planet_ink.coffee_mud.Commands.interfaces.*;
import com.planet_ink.coffee_mud.Common.interfaces.*;
import com.planet_ink.coffee_mud.Common.interfaces.CharStats.CODES;
import com.planet_ink.coffee_mud.Exits.interfaces.*;
import com.planet_ink.coffee_mud.Items.interfaces.*;
import com.planet_ink.coffee_mud.Locales.interfaces.*;
import com.planet_ink.coffee_mud.MOBS.interfaces.*;
import com.planet_ink.coffee_mud.Races.interfaces.*;

/* 
   Copyright 2000-2013 Bo Zimmerman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

	   http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/
public interface RawMaterial extends Item
{
	
	public String domainSource();
	public void setDomainSource(String src);
	public boolean rebundle();
	public void quickDestroy();
	
	// item materials
	public final static int MATERIAL_UNKNOWN=0;
	public final static int MATERIAL_CLOTH=1<<8;
	public final static int MATERIAL_LEATHER=2<<8;
	public final static int MATERIAL_METAL=3<<8;
	public final static int MATERIAL_MITHRIL=4<<8;
	public final static int MATERIAL_WOODEN=5<<8;
	public final static int MATERIAL_GLASS=6<<8;
	public final static int MATERIAL_VEGETATION=7<<8;
	public final static int MATERIAL_FLESH=8<<8;
	public final static int MATERIAL_PAPER=9<<8;
	public final static int MATERIAL_ROCK=10<<8;
	public final static int MATERIAL_LIQUID=11<<8;
	public final static int MATERIAL_PRECIOUS=12<<8;
	public final static int MATERIAL_ENERGY=13<<8;
	public final static int MATERIAL_SYNTHETIC=14<<8;
	public final static int MATERIAL_GAS=15<<8;

	public final static int MATERIAL_MASK=255<<8;
	
	public static enum Material
	{
		UNKNOWN("Unknown material"),
		CLOTH("Cloth"),
		LEATHER("Leather"),
		METAL("Metal"),
		MITHRIL("Metal"),
		WOODEN("Wood"),
		GLASS("Glass"),
		VEGETATION("Vegetation"),
		FLESH("Flesh"),
		PAPER("Paper"),
		ROCK("Rock"),
		LIQUID("Liquid"),
		PRECIOUS("Stone"),
		ENERGY("Energy"),
		SYNTHETIC("Plastic"),
		GAS("Gas")
		;
		private final static String[] descs=CMParms.toStringArray(values());
		private final int mask;
		private final String desc;
		private final String noun;
		private Material(String noun)
		{
			this.mask=(ordinal()==0)?0:(ordinal()<<8);
			this.desc=this.toString();
			this.noun=noun;
		}
		public static int size() { return values().length; }
		public int mask() { return mask; }
		public String noun() { return noun; }
		public String desc() { return desc; }
		public static String[] names() { return descs; }
		public static Material findByMask(int mask)
		{
			int maskOrdinal=(mask<size())?mask:(mask&MATERIAL_MASK)>>8;
			if(maskOrdinal < Material.values().length)
				return Material.values()[maskOrdinal];
			return null;
		}
		public static Material find(String name)
		{
			try {
				return Material.valueOf(name);
			} catch(Exception e) {
				return null;
			}
		}
		public static Material findIgnoreCase(String name)
		{
			return find(name.toUpperCase().trim());
		}
		public static Material startsWith(String name)
		{
			for(Material m : values())
				if(m.desc.startsWith(name))
					return m;
			return null;
		}
		public static Material startsWithIgnoreCase(String name)
		{
			return startsWith(name.toUpperCase().trim());
		}
	}
	
	public final static int RESOURCE_NOTHING=MATERIAL_UNKNOWN|0;
	public final static int RESOURCE_MEAT=MATERIAL_FLESH|1;
	public final static int RESOURCE_BEEF=MATERIAL_FLESH|2;
	public final static int RESOURCE_PORK=MATERIAL_FLESH|3;
	public final static int RESOURCE_POULTRY=MATERIAL_FLESH|4;
	public final static int RESOURCE_MUTTON=MATERIAL_FLESH|5;
	public final static int RESOURCE_FISH=MATERIAL_FLESH|6;
	public final static int RESOURCE_WHEAT=MATERIAL_VEGETATION|7;
	public final static int RESOURCE_CORN=MATERIAL_VEGETATION|8;
	public final static int RESOURCE_RICE=MATERIAL_VEGETATION|9;
	public final static int RESOURCE_CARROTS=MATERIAL_VEGETATION|10;
	public final static int RESOURCE_TOMATOES=MATERIAL_VEGETATION|11;
	public final static int RESOURCE_PEPPERS=MATERIAL_VEGETATION|12;
	public final static int RESOURCE_GREENS=MATERIAL_VEGETATION|13;
	public final static int RESOURCE_FRUIT=MATERIAL_VEGETATION|14;
	public final static int RESOURCE_APPLES=MATERIAL_VEGETATION|15;
	public final static int RESOURCE_BERRIES=MATERIAL_VEGETATION|16;
	public final static int RESOURCE_ORANGES=MATERIAL_VEGETATION|17;
	public final static int RESOURCE_LEMONS=MATERIAL_VEGETATION|18;
	public final static int RESOURCE_GRAPES=MATERIAL_VEGETATION|19;
	public final static int RESOURCE_OLIVES=MATERIAL_VEGETATION|20;
	public final static int RESOURCE_POTATOES=MATERIAL_VEGETATION|21;
	public final static int RESOURCE_CACTUS=MATERIAL_VEGETATION|22;
	public final static int RESOURCE_DATES=MATERIAL_VEGETATION|23;
	public final static int RESOURCE_SEAWEED=MATERIAL_VEGETATION|24;
	public final static int RESOURCE_STONE=MATERIAL_ROCK|25;
	public final static int RESOURCE_LIMESTONE=MATERIAL_ROCK|26;
	public final static int RESOURCE_FLINT=MATERIAL_ROCK|27;
	public final static int RESOURCE_GRANITE=MATERIAL_ROCK|28;
	public final static int RESOURCE_OBSIDIAN=MATERIAL_ROCK|29;
	public final static int RESOURCE_MARBLE=MATERIAL_ROCK|30;
	public final static int RESOURCE_SAND=MATERIAL_ROCK|31;
	public final static int RESOURCE_JADE=MATERIAL_PRECIOUS|32;
	public final static int RESOURCE_IRON=MATERIAL_METAL|33;
	public final static int RESOURCE_LEAD=MATERIAL_METAL|34;
	public final static int RESOURCE_BRONZE=MATERIAL_METAL|35;
	public final static int RESOURCE_SILVER=MATERIAL_METAL|36;
	public final static int RESOURCE_GOLD=MATERIAL_METAL|37;
	public final static int RESOURCE_ZINC=MATERIAL_METAL|38;
	public final static int RESOURCE_COPPER=MATERIAL_METAL|39;
	public final static int RESOURCE_TIN=MATERIAL_METAL|40;
	public final static int RESOURCE_MITHRIL=MATERIAL_MITHRIL|41;
	public final static int RESOURCE_ADAMANTITE=MATERIAL_MITHRIL|42;
	public final static int RESOURCE_STEEL=MATERIAL_METAL|43;
	public final static int RESOURCE_BRASS=MATERIAL_METAL|44;
	public final static int RESOURCE_WOOD=MATERIAL_WOODEN|45;
	public final static int RESOURCE_PINE=MATERIAL_WOODEN|46;
	public final static int RESOURCE_BALSA=MATERIAL_WOODEN|47;
	public final static int RESOURCE_OAK=MATERIAL_WOODEN|48;
	public final static int RESOURCE_MAPLE=MATERIAL_WOODEN|49;
	public final static int RESOURCE_REDWOOD=MATERIAL_WOODEN|50;
	public final static int RESOURCE_HICKORY=MATERIAL_WOODEN|51;
	public final static int RESOURCE_SCALES=MATERIAL_LEATHER|52;
	public final static int RESOURCE_FUR=MATERIAL_CLOTH|53;
	public final static int RESOURCE_LEATHER=MATERIAL_LEATHER|54;
	public final static int RESOURCE_HIDE=MATERIAL_CLOTH|55;
	public final static int RESOURCE_WOOL=MATERIAL_CLOTH|56;
	public final static int RESOURCE_FEATHERS=MATERIAL_CLOTH|57;
	public final static int RESOURCE_COTTON=MATERIAL_CLOTH|58;
	public final static int RESOURCE_HEMP=MATERIAL_CLOTH|59;
	public final static int RESOURCE_FRESHWATER=MATERIAL_LIQUID|60;
	public final static int RESOURCE_SALTWATER=MATERIAL_LIQUID|61;
	public final static int RESOURCE_DRINKABLE=MATERIAL_LIQUID|62;
	public final static int RESOURCE_GLASS=MATERIAL_GLASS|63;
	public final static int RESOURCE_PAPER=MATERIAL_PAPER|64;
	public final static int RESOURCE_CLAY=MATERIAL_GLASS|65;
	public final static int RESOURCE_CHINA=MATERIAL_GLASS|66;
	public final static int RESOURCE_DIAMOND=MATERIAL_PRECIOUS|67;
	public final static int RESOURCE_CRYSTAL=MATERIAL_GLASS|68;
	public final static int RESOURCE_GEM=MATERIAL_PRECIOUS|69;
	public final static int RESOURCE_PEARL=MATERIAL_PRECIOUS|70;
	public final static int RESOURCE_PLATINUM=MATERIAL_METAL|71;
	public final static int RESOURCE_MILK=MATERIAL_LIQUID|72;
	public final static int RESOURCE_EGGS=MATERIAL_FLESH|73;
	public final static int RESOURCE_HOPS=MATERIAL_VEGETATION|74;
	public final static int RESOURCE_COFFEEBEANS=MATERIAL_VEGETATION|75;
	public final static int RESOURCE_COFFEE=MATERIAL_LIQUID|76;
	public final static int RESOURCE_OPAL=MATERIAL_PRECIOUS|77;
	public final static int RESOURCE_TOPAZ=MATERIAL_PRECIOUS|78;
	public final static int RESOURCE_AMETHYST=MATERIAL_PRECIOUS|79;
	public final static int RESOURCE_GARNET=MATERIAL_PRECIOUS|80;
	public final static int RESOURCE_AMBER=MATERIAL_PRECIOUS|81;
	public final static int RESOURCE_AQUAMARINE=MATERIAL_PRECIOUS|82;
	public final static int RESOURCE_CRYSOBERYL=MATERIAL_PRECIOUS|83;
	public final static int RESOURCE_IRONWOOD=MATERIAL_WOODEN|84;
	public final static int RESOURCE_SILK=MATERIAL_CLOTH|85;
	public final static int RESOURCE_COCOA=MATERIAL_VEGETATION|86;
	public final static int RESOURCE_BLOOD=MATERIAL_LIQUID|87;
	public final static int RESOURCE_BONE=MATERIAL_ROCK|88;
	public final static int RESOURCE_COAL=MATERIAL_ROCK|89;
	public final static int RESOURCE_LAMPOIL=MATERIAL_LIQUID|90;
	public final static int RESOURCE_POISON=MATERIAL_LIQUID|91;
	public final static int RESOURCE_LIQUOR=MATERIAL_LIQUID|92;
	public final static int RESOURCE_SUGAR=MATERIAL_VEGETATION|93;
	public final static int RESOURCE_HONEY=MATERIAL_LIQUID|94;
	public final static int RESOURCE_BARLEY=MATERIAL_VEGETATION|95;
	public final static int RESOURCE_MUSHROOMS=MATERIAL_VEGETATION|96;
	public final static int RESOURCE_HERBS=MATERIAL_VEGETATION|97;
	public final static int RESOURCE_VINE=MATERIAL_VEGETATION|98;
	public final static int RESOURCE_FLOWERS=MATERIAL_VEGETATION|99;
	public final static int RESOURCE_PLASTIC=MATERIAL_SYNTHETIC|100;
	public final static int RESOURCE_RUBBER=MATERIAL_SYNTHETIC|101;
	public final static int RESOURCE_EBONY=MATERIAL_ROCK|102;
	public final static int RESOURCE_IVORY=MATERIAL_ROCK|103;
	public final static int RESOURCE_WAX=MATERIAL_FLESH|104;
	public final static int RESOURCE_NUTS=MATERIAL_VEGETATION|105;
	public final static int RESOURCE_BREAD=MATERIAL_VEGETATION|106;
	public final static int RESOURCE_CRACKER=MATERIAL_VEGETATION|107;
	public final static int RESOURCE_YEW=MATERIAL_WOODEN|108;
	public final static int RESOURCE_DUST=MATERIAL_ROCK|109;
	public final static int RESOURCE_PIPEWEED=MATERIAL_VEGETATION|110;
	public final static int RESOURCE_ENERGY=MATERIAL_ENERGY|111;
	public final static int RESOURCE_STRAWBERRIES=MATERIAL_VEGETATION|112;
	public final static int RESOURCE_BLUEBERRIES=MATERIAL_VEGETATION|113;
	public final static int RESOURCE_RASPBERRIES=MATERIAL_VEGETATION|114;
	public final static int RESOURCE_BOYSENBERRIES=MATERIAL_VEGETATION|115;
	public final static int RESOURCE_BLACKBERRIES=MATERIAL_VEGETATION|116;
	public final static int RESOURCE_SMURFBERRIES=MATERIAL_VEGETATION|117;
	public final static int RESOURCE_PEACHES=MATERIAL_VEGETATION|118;
	public final static int RESOURCE_PLUMS=MATERIAL_VEGETATION|119;
	public final static int RESOURCE_ONIONS=MATERIAL_VEGETATION|120;
	public final static int RESOURCE_CHERRIES=MATERIAL_VEGETATION|121;
	public final static int RESOURCE_GARLIC=MATERIAL_VEGETATION|122;
	public final static int RESOURCE_PINEAPPLES=MATERIAL_VEGETATION|123;
	public final static int RESOURCE_COCONUTS=MATERIAL_VEGETATION|124;   
	public final static int RESOURCE_BANANAS=MATERIAL_VEGETATION|125;   
	public final static int RESOURCE_LIMES=MATERIAL_VEGETATION|126;
	public final static int RESOURCE_SAP=MATERIAL_LIQUID|127;      
	public final static int RESOURCE_ONYX=MATERIAL_PRECIOUS|128;
	public final static int RESOURCE_TURQUOISE=MATERIAL_PRECIOUS|129;
	public final static int RESOURCE_PERIDOT=MATERIAL_PRECIOUS|130;
	public final static int RESOURCE_QUARTZ=MATERIAL_PRECIOUS|131;
	public final static int RESOURCE_LAPIS=MATERIAL_PRECIOUS|132;
	public final static int RESOURCE_BLOODSTONE=MATERIAL_PRECIOUS|133;
	public final static int RESOURCE_MOONSTONE=MATERIAL_PRECIOUS|134;
	public final static int RESOURCE_ALEXANDRITE=MATERIAL_PRECIOUS|135;
	public final static int RESOURCE_TEAK=MATERIAL_WOODEN|136;
	public final static int RESOURCE_CEDAR=MATERIAL_WOODEN|137;
	public final static int RESOURCE_ELM=MATERIAL_WOODEN|138;
	public final static int RESOURCE_CHERRYWOOD=MATERIAL_WOODEN|139;
	public final static int RESOURCE_BEECHWOOD=MATERIAL_WOODEN|140;
	public final static int RESOURCE_WILLOW=MATERIAL_WOODEN|141;
	public final static int RESOURCE_SYCAMORE=MATERIAL_WOODEN|142;
	public final static int RESOURCE_SPRUCE=MATERIAL_WOODEN|143;
	public final static int RESOURCE_MESQUITE=MATERIAL_WOODEN|144;
	public final static int RESOURCE_BASALT=MATERIAL_ROCK|145;
	public final static int RESOURCE_SHALE=MATERIAL_ROCK|146;
	public final static int RESOURCE_PUMICE=MATERIAL_ROCK|147;
	public final static int RESOURCE_SANDSTONE=MATERIAL_ROCK|148;   
	public final static int RESOURCE_SOAPSTONE=MATERIAL_ROCK|149;   
	public final static int RESOURCE_SALMON=MATERIAL_FLESH|150;
	public final static int RESOURCE_CARP=MATERIAL_FLESH|151;
	public final static int RESOURCE_TROUT=MATERIAL_FLESH|152;
	public final static int RESOURCE_SHRIMP=MATERIAL_FLESH|153;
	public final static int RESOURCE_TUNA=MATERIAL_FLESH|154;
	public final static int RESOURCE_CATFISH=MATERIAL_FLESH|155;
	public final static int RESOURCE_BAMBOO=MATERIAL_WOODEN|156;
	public final static int RESOURCE_SOAP=MATERIAL_VEGETATION|157;
	public final static int RESOURCE_SPIDERSTEEL=MATERIAL_CLOTH|158;
	public final static int RESOURCE_ASH=MATERIAL_VEGETATION|159;
	public final static int RESOURCE_PERFUME=MATERIAL_LIQUID|160;
	public final static int RESOURCE_ATLANTITE=MATERIAL_MITHRIL|161;
	public final static int RESOURCE_CHEESE=MATERIAL_VEGETATION|162;
	public final static int RESOURCE_BEANS=MATERIAL_VEGETATION|163;
	public final static int RESOURCE_CRANBERRIES=MATERIAL_VEGETATION|164;
	public final static int RESOURCE_DRAGONBLOOD=MATERIAL_LIQUID|165;
	public final static int RESOURCE_DRAGONMEAT=MATERIAL_FLESH|166;
	public final static int RESOURCE_RUBY=MATERIAL_PRECIOUS|167;
	public final static int RESOURCE_EMERALD=MATERIAL_PRECIOUS|168;
	public final static int RESOURCE_SAPPHIRE=MATERIAL_PRECIOUS|169;
	public final static int RESOURCE_AGATE=MATERIAL_PRECIOUS|170;
	public final static int RESOURCE_CITRINE=MATERIAL_PRECIOUS|171;
	public final static int RESOURCE_REED=MATERIAL_WOODEN|172;
	public final static int RESOURCE_ALABASTER=MATERIAL_ROCK|173;
	public final static int RESOURCE_CHROMIUM=MATERIAL_METAL|174;
	public final static int RESOURCE_WHITE_GOLD=MATERIAL_METAL|175;
	public final static int RESOURCE_DRAGONSCALES=MATERIAL_METAL|176;
	public final static int RESOURCE_PLASMA=MATERIAL_ENERGY|177;
	public final static int RESOURCE_ELECTRICITY=MATERIAL_ENERGY|178;
	public final static int RESOURCE_DEUTERIUM=MATERIAL_SYNTHETIC|179;
	public final static int RESOURCE_SLIME=MATERIAL_LIQUID|180;
	public final static int RESOURCE_AIR=MATERIAL_GAS|181;
	public final static int RESOURCE_OXYGEN=MATERIAL_GAS|182;
	public final static int RESOURCE_HYDROGEN=MATERIAL_GAS|183;
	public final static int RESOURCE_FLOURINE=MATERIAL_GAS|184;
	public final static int RESOURCE_NITROGEN=MATERIAL_GAS|185;
	public final static int RESOURCE_MASK=255;

	
	public static enum ResourceFlag
	{
		BERRY,
		FISH
	}
	
	public static enum DefResource
	{
		NOTHING(RESOURCE_NOTHING,0,0,0,0,"","",null),
		MEAT(RESOURCE_MEAT,4,20,1,3000,"","",null),
		BEEF(RESOURCE_BEEF,6,20,1,3000,"","",null),
		PORK(RESOURCE_PORK,8,20,1,2500,"","",null),
		POULTRY(RESOURCE_POULTRY,3,20,1,2000,"","",null),
		MUTTON(RESOURCE_MUTTON,4,20,1,2800,"","",null),
		FISH(RESOURCE_FISH,5,100,1,590,"strong fishy","",ResourceFlag.FISH),
		WHEAT(RESOURCE_WHEAT,1,20,1,770,"","",null),
		CORN(RESOURCE_CORN,1,20,1,720,"","",null),
		RICE(RESOURCE_RICE,1,20,1,750,"","",null),
		CARROTS(RESOURCE_CARROTS,1,5,1,720,"","",null),
		TOMATOES(RESOURCE_TOMATOES,1,5,1,640,"","",null),
		PEPPERS(RESOURCE_PEPPERS,1,5,1,640,"spicy","",null),
		GREENS(RESOURCE_GREENS,1,5,1,540,"very mild","",null),
		FRUIT(RESOURCE_FRUIT,2,10,1,720,"sweet and fruity","",null),
		APPLES(RESOURCE_APPLES,2,10,1,640,"sweet apply","",null),
		BERRIES(RESOURCE_BERRIES,2,15,1,720,"sweet berry","",ResourceFlag.BERRY),
		ORANGES(RESOURCE_ORANGES,2,10,1,640,"citrusy","",null),
		LEMONS(RESOURCE_LEMONS,2,10,1,480,"strong citrusy","Prop_UseSpellCast2(Spell_ShrinkMouth)",null),
		GRAPES(RESOURCE_GRAPES,3,5,1,680,"mild sweet","",null),
		OLIVES(RESOURCE_OLIVES,2,5,1,640,"pickly olive","",null),
		POTATOES(RESOURCE_POTATOES,1,5,1,770,"","",null),
		CACTUS(RESOURCE_CACTUS,2,5,1,680,"","",null),
		DATES(RESOURCE_DATES,2,2,1,720,"sweet plumy","",null),
		SEAWEED(RESOURCE_SEAWEED,1,50,1,540,"","",null),
		STONE(RESOURCE_STONE,1,80,5,2500,"mild musty","",null),
		LIMESTONE(RESOURCE_LIMESTONE,1,20,4,1550,"","",null),
		FLINT(RESOURCE_FLINT,1,10,4,2600,"","",null),
		GRANITE(RESOURCE_GRANITE,2,10,6,2690,"","",null),
		OBSIDIAN(RESOURCE_OBSIDIAN,10,5,6,2650,"","",null),
		MARBLE(RESOURCE_MARBLE,20,5,5,2560,"","",null),
		SAND(RESOURCE_SAND,1,50,1,1600,"","",null),
		JADE(RESOURCE_JADE,50,2,5,3800,"","",null),
		IRON(RESOURCE_IRON,20,10,6,7900,"","",null),
		LEAD(RESOURCE_LEAD,10,10,4,11300,"","",null),
		BRONZE(RESOURCE_BRONZE,10,10,5,8100,"","",null),
		SILVER(RESOURCE_SILVER,30,2,5,10500,"","",null),
		GOLD(RESOURCE_GOLD,150,1,5,19320,"","",null),
		ZINC(RESOURCE_ZINC,10,5,5,7100,"","",null),
		COPPER(RESOURCE_COPPER,10,10,5,8900,"","",null),
		TIN(RESOURCE_TIN,10,10,4,7300,"","",null),
		MITHRIL(RESOURCE_MITHRIL,100,1,9,3990,"","",null),
		ADAMANTITE(RESOURCE_ADAMANTITE,175,1,10,4500,"","",null),
		STEEL(RESOURCE_STEEL,75,0,8,7840,"","",null),
		BRASS(RESOURCE_BRASS,120,0,6,5500,"","",null),
		WOOD(RESOURCE_WOOD,2,10,3,920,"","",null),
		PINE(RESOURCE_PINE,4,10,3,650,"fresh, clean piney","",null),
		BALSA(RESOURCE_BALSA,1,5,2,130,"","",null),
		OAK(RESOURCE_OAK,5,10,3,720,"rich oaky","",null),
		MAPLE(RESOURCE_MAPLE,10,5,3,689,"mild maply","",null),
		REDWOOD(RESOURCE_REDWOOD,20,2,3,450,"","",null),
		HICKORY(RESOURCE_HICKORY,5,5,3,830,"","",null),
		SCALES(RESOURCE_SCALES,10,20,4,1800,"","",null),
		FUR(RESOURCE_FUR,20,20,2,890,"musky","",null),
		LEATHER(RESOURCE_LEATHER,10,20,2,945,"strong leathery","",null),
		HIDE(RESOURCE_HIDE,4,20,2,920,"mild stinky","",null),
		WOOL(RESOURCE_WOOL,10,20,1,1310,"","",null),
		FEATHERS(RESOURCE_FEATHERS,10,20,1,20,"","",null),
		COTTON(RESOURCE_COTTON,5,20,1,590,"","",null),
		HEMP(RESOURCE_HEMP,4,10,1,720,"grassy","",null),
		WATER(RESOURCE_FRESHWATER,0,100,0,1000,"","",null),
		SALT_WATER(RESOURCE_SALTWATER,0,100,0,1030,"","",null),
		LIQUID(RESOURCE_DRINKABLE,0,1,0,1000,"","",null),
		GLASS(RESOURCE_GLASS,10,0,3,2800,"","",null),
		PAPER(RESOURCE_PAPER,10,0,0,920,"","",null),
		CLAY(RESOURCE_CLAY,1,50,1,1750,"mild dusty","",null),
		CHINA(RESOURCE_CHINA,30,0,3,2400,"","",null),
		DIAMOND(RESOURCE_DIAMOND,500,1,9,3510,"","",null),
		CRYSTAL(RESOURCE_CRYSTAL,10,5,3,2200,"","",null),
		GEM(RESOURCE_GEM,100,1,3,3500,"","",null),
		PEARL(RESOURCE_PEARL,380,1,4,2000,"","",null),
		PLATINUM(RESOURCE_PLATINUM,80,1,6,21450,"","",null),
		MILK(RESOURCE_MILK,2,10,0,1020,"mild milky","",null),
		EGGS(RESOURCE_EGGS,2,10,0,1120,"","",null),
		HOPS(RESOURCE_HOPS,2,20,1,340,"mild grainy","",null),
		COFFEEBEANS(RESOURCE_COFFEEBEANS,2,10,1,560,"mild coffee","Poison_Caffeine",null),
		COFFEE(RESOURCE_COFFEE,0,10,0,430,"rich coffee","Poison_Caffeine",null),
		OPAL(RESOURCE_OPAL,80,2,5,2250,"","",null),
		TOPAZ(RESOURCE_TOPAZ,200,2,5,3570,"","",null),
		AMETHYST(RESOURCE_AMETHYST,300,2,5,2651,"","",null),
		GARNET(RESOURCE_GARNET,70,2,5,3870,"","",null),
		AMBER(RESOURCE_AMBER,80,5,5,2500,"","",null),
		AQUAMARINE(RESOURCE_AQUAMARINE,50,2,5,2800,"","",null),
		CRYSOBERYL(RESOURCE_CRYSOBERYL,50,2,5,2800,"","",null),
		IRONWOOD(RESOURCE_IRONWOOD,25,5,4,990,"","",null),
		SILK(RESOURCE_SILK,200,5,1,1100,"","",null),
		COCOA(RESOURCE_COCOA,4,5,0,590,"","",null),
		BLOOD(RESOURCE_BLOOD,1,100,0,1025,"strong salty","",null),
		BONE(RESOURCE_BONE,1,100,5,1600,"","",null),
		COAL(RESOURCE_COAL,1,50,1,1800,"chalky","",null),
		LAMP_OIL(RESOURCE_LAMPOIL,1,10,1,880,"light oily","L",null),
		POISON(RESOURCE_POISON,1,1,1,1000,"","",null),
		LIQUOR(RESOURCE_LIQUOR,10,1,1,790,"alcohol","",null),
		SUGAR(RESOURCE_SUGAR,1,50,1,1600,"","",null),
		HONEY(RESOURCE_HONEY,1,50,1,1600,"","",null),
		BARLEY(RESOURCE_BARLEY,1,20,1,610,"","",null),
		MUSHROOMS(RESOURCE_MUSHROOMS,1,20,1,500,"","",null),
		HERBS(RESOURCE_HERBS,1,10,1,770,"fresh herbal","",null),
		VINE(RESOURCE_VINE,1,10,1,880,"rich green","",null),
		FLOWERS(RESOURCE_FLOWERS,1,10,1,720,"nice floral","",null),
		PLASTIC(RESOURCE_PLASTIC,25,0,4,950,"","",null),
		RUBBER(RESOURCE_RUBBER,25,0,1,1506,"sour rubbery","",null),
		EBONY(RESOURCE_EBONY,5,5,5,2910,"","",null),
		IVORY(RESOURCE_IVORY,5,5,3,1840,"","",null),
		WAX(RESOURCE_WAX,1,0,0,900,"","",null),
		NUTS(RESOURCE_NUTS,0,20,0,640,"mild nutty","",null),
		BREAD(RESOURCE_BREAD,3,0,0,660,"","",null),
		CRACKER(RESOURCE_CRACKER,2,0,0,200,"","",null),
		YEW(RESOURCE_YEW,15,2,5,850,"","",null),
		DUST(RESOURCE_DUST,0,20,0,1120,"dusty","",null),
		PIPEWEED(RESOURCE_PIPEWEED,3,10,1,320,"strong grassy","",null),
		ENERGY(RESOURCE_ENERGY,30,0,4,0,"","",null),
		STRAWBERRIES(RESOURCE_STRAWBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		BLUEBERRIES(RESOURCE_BLUEBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		RASPBERRIES(RESOURCE_RASPBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		BOYSENBERRIES(RESOURCE_BOYSENBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		BLACKBERRIES(RESOURCE_BLACKBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		SMURFBERRIES(RESOURCE_SMURFBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		PEACHES(RESOURCE_PEACHES,10,1,1,700,"peachy","",null),
		PLUMS(RESOURCE_PLUMS,10,1,1,710,"sweey plumy","",null),
		ONIONS(RESOURCE_ONIONS,10,1,1,760,"stinging oniony","",null),
		CHERRIES(RESOURCE_CHERRIES,10,1,1,810,"cherry","",null),
		GARLIC(RESOURCE_GARLIC,10,1,1,815,"","",null),
		PINEAPPLES(RESOURCE_PINEAPPLES,10,1,1,500,"fruity","",null),
		COCONUTS(RESOURCE_COCONUTS,10,1,2,250,"","",null),
		BANANAS(RESOURCE_BANANAS,10,1,1,790,"pungent banana","",null),
		LIMES(RESOURCE_LIMES,10,1,1,690,"citrusy","",null),
		SAP(RESOURCE_SAP,10,1,1,1600,"strong maply","",null),
		ONYX(RESOURCE_ONYX,70,1,8,3300,"","",null),
		TURQUOISE(RESOURCE_TURQUOISE,70,1,8,3300,"","",null),
		PERIDOT(RESOURCE_PERIDOT,65,1,6,3300,"","",null),
		QUARTZ(RESOURCE_QUARTZ,25,1,5,3300,"","",null),
		LAPIS(RESOURCE_LAPIS,70,1,6,3300,"","",null),
		BLOODSTONE(RESOURCE_BLOODSTONE,85,1,8,3300,"","",null),
		MOONSTONE(RESOURCE_MOONSTONE,90,1,8,3300,"","",null),
		ALEXANDRITE(RESOURCE_ALEXANDRITE,95,1,9,3300,"","",null),
		TEAK(RESOURCE_TEAK,20,2,3,1000,"","",null),
		CEDAR(RESOURCE_CEDAR,15,2,3,900,"strong cedar","",null),
		ELM(RESOURCE_ELM,15,2,3,1100,"","",null),
		CHERRYWOOD(RESOURCE_CHERRYWOOD,17,2,3,900,"","",null),
		BEECHWOOD(RESOURCE_BEECHWOOD,12,2,3,975,"","",null),
		WILLOW(RESOURCE_WILLOW,12,2,1,1000,"","",null),
		SYCAMORE(RESOURCE_SYCAMORE,11,2,2,1000,"","",null),
		SPRUCE(RESOURCE_SPRUCE,12,2,3,990,"","",null),
		MESQUITE(RESOURCE_MESQUITE,9,2,3,1150,"rich mesquite","",null),
		BASALT(RESOURCE_BASALT,10,2,4,3300,"","",null),
		SHALE(RESOURCE_SHALE,5,2,2,1200,"","",null),
		PUMICE(RESOURCE_PUMICE,5,2,4,600,"","",null),
		SANDSTONE(RESOURCE_SANDSTONE,10,2,2,3500,"","",null),
		SOAPSTONE(RESOURCE_SOAPSTONE,60,2,5,3600,"","",null),
		SALMON(RESOURCE_SALMON,6,1,1,1000,"strong fishy","",ResourceFlag.FISH),
		CARP(RESOURCE_CARP,6,1,1,1000,"strong fishy","",ResourceFlag.FISH),
		TROUT(RESOURCE_TROUT,6,1,1,1000,"strong fishy","",ResourceFlag.FISH),
		SHRIMP(RESOURCE_SHRIMP,6,1,1,1000,"mild fishy","",ResourceFlag.FISH),
		TUNA(RESOURCE_TUNA,6,1,1,1000,"strong fishy","",ResourceFlag.FISH),
		CATFISH(RESOURCE_CATFISH,6,1,1,1000,"strong fishy","",ResourceFlag.FISH),
		BAMBOO(RESOURCE_BAMBOO,15,10,4,120,"","",null),
		SOAP(RESOURCE_SOAP,1,0,1,430,"light fragrant","",null),
		SPIDERSTEEL(RESOURCE_SPIDERSTEEL,150,0,2,630,"","",null),
		ASH(RESOURCE_ASH,1,0,0,50,"dusty","",null),
		PERFUME(RESOURCE_PERFUME,1,1,1,1000,"strong fragrant","",null),
		ATLANTITE(RESOURCE_ATLANTITE,200,1,6,850,"","",null),
		CHEESE(RESOURCE_CHEESE,25,0,1,640,"mild cheesy","",null),
		BEANS(RESOURCE_BEANS,1,15,1,750,"","",null),
		CRANBERRIES(RESOURCE_CRANBERRIES,10,1,1,750,"sweet berry","",ResourceFlag.BERRY),
		DRAGONBLOOD(RESOURCE_DRAGONBLOOD,10,100,0,1025,"mild salty","Prop_UseSpellCast2(Prayer_MinorInfusion)",null),
		DRAGONMEAT(RESOURCE_DRAGONMEAT,40,20,1,3000,"mild salty","",null),
		RUBY(RESOURCE_RUBY,300,1,3,3500,"","",null),
		EMERALD(RESOURCE_EMERALD,200,1,3,3500,"","",null),
		SAPPHIRE(RESOURCE_SAPPHIRE,180,1,3,3500,"","",null),
		AGATE(RESOURCE_AGATE,100,1,3,3500,"","",null),
		CITRINE(RESOURCE_CITRINE,100,1,3,3500,"","",null),
		REED(RESOURCE_REED,1,50,3,830,"","",null),
		ALABASTER(RESOURCE_ALABASTER,5,10,4,1550,"","",null),
		CHROMIUM(RESOURCE_CHROMIUM,80,1,6,1550,"","",null),
		WHITE_GOLD(RESOURCE_WHITE_GOLD,150,1,5,1450,"","",null),
		DRAGONSCALES(RESOURCE_DRAGONSCALES,50,1,6,800,"","",null),
		PLASMA(RESOURCE_PLASMA,100,0,0,0,"","",null),
		ELECTRICITY(RESOURCE_ELECTRICITY,100,0,0,0,"","",null),
		DEUTERIUM(RESOURCE_DEUTERIUM,100,0,0,0,"","",null),
		SLIME(RESOURCE_SLIME,5,10,1,750,"horridly acidic","Prop_UseSpellCast2(Spell_AcidSpray)",null),
		AIR(RESOURCE_AIR,0,100,0,0,"","",null),
		OXYGEN(RESOURCE_OXYGEN,10,100,0,0,"","",null),
		HYDROGEN(RESOURCE_HYDROGEN,20,100,0,0,"","",null),
		FLOURINE(RESOURCE_FLOURINE,60,40,0,0,"strong pungent smell","",null),
		NITROGEN(RESOURCE_NITROGEN,10,100,0,0,"","",null)
		;
		public final int code, value, frequency, hardness, bouancy;
		public final String smell, effect, desc;
		public final ResourceFlag flag;
		private DefResource(int cod, int val, int freq, int hard, int bou, String sniff, String eff, ResourceFlag flg)
		{
			code=cod; value=val; frequency=freq;
			hardness=hard; bouancy=bou;
			smell=sniff; effect=eff; flag=flg;
			desc=toString().replace('_', ' ');
		}
	}
	
	/**
	 * Global resource/raw material code data collector
	 * @author bzimmerman
	 */
	public class CODES
	{
		@SuppressWarnings("rawtypes")
		public CODES(){
			super();
			char c=Thread.currentThread().getThreadGroup().getName().charAt(0);
			if(insts==null) insts=new CODES[256];
			if(insts[c]==null) insts[c]=this;
			synchronized(this)
			{
				String[][] addExtra = CMProps.instance().getStrsStarting("ADDMATERIAL_");
				String[][] repExtra = CMProps.instance().getStrsStarting("REPLACEMATERIAL_");
				
				for(DefResource d : DefResource.values())
				{
					int material= d.code & MATERIAL_MASK;
					add(material, d.desc, d.smell, d.value, d.frequency, d.hardness, d.bouancy, 
							d.flag==ResourceFlag.FISH,d.flag==ResourceFlag.BERRY,d.effect);
				}
				for(int i=0;i<addExtra.length + repExtra.length;i++)
				{
					String[] array = (i>=addExtra.length)?repExtra[i-addExtra.length]:addExtra[i];
					boolean replace = i>=addExtra.length;
					String stat = array[0].toUpperCase().trim();
					String p=array[1];
					Vector V=CMParms.parseCommas(p, false);
					if(V.size()!=8) {
						Log.errOut("RawMaterial","Bad coffeemud.ini material row (requires 8 elements, separated by ,): "+p);
						continue;
					}
					String type="ADD";
					int oldResourceCode=-1;
					if(replace)
					{
						DefResource r=(DefResource)CMath.s_valueOf(DefResource.class, stat.toUpperCase().trim().replace(' ','_'));
						if(r!=null)
						{
							oldResourceCode=r.code;
							type="REPLACE";
						}
						else
						{
							Log.errOut("RawMaterial","Unknown replaceable resource in coffeemud.ini: "+stat);
							continue;
						}
					}
					String matStr=((String)V.elementAt(0)).toUpperCase();
					String smell=((String)V.elementAt(1)).toUpperCase();
					int value=CMath.s_int((String)V.elementAt(2));
					int frequ=CMath.s_int((String)V.elementAt(3));
					int hardness=CMath.s_int((String)V.elementAt(4));
					int bouancy=CMath.s_int((String)V.elementAt(5));
					boolean fish=((String)V.elementAt(6)).equalsIgnoreCase("fish");
					boolean berry=((String)V.elementAt(6)).equalsIgnoreCase("berry");
					String abilityID=(String)V.elementAt(7);
					Material material=Material.findIgnoreCase(matStr);
					if(material==null) 
					{
						Log.errOut("RawMaterial","Unknown material code in coffeemud.ini: "+matStr);
						continue;
					}
					if(type.equalsIgnoreCase("ADD"))
						add(material.mask(), stat, smell, value, frequ, hardness, bouancy, fish, berry, abilityID);
					else
					if(type.equalsIgnoreCase("REPLACE")&&(oldResourceCode>=0))
						replace(oldResourceCode, material.mask(), stat, smell, value, frequ, hardness, bouancy, fish, berry, abilityID);
				}
				String[] sortedNames = descs.clone();
				Arrays.sort(sortedNames);
				Hashtable<String,Integer> previousIndexes = new Hashtable<String,Integer>();
				for(int ndex = 0; ndex < descs.length; ndex++)
					previousIndexes.put(descs[ndex], Integer.valueOf(ndex));
				allCodesSortedByName = new int[allCodes.length];
				for(int ndex = 0; ndex < sortedNames.length; ndex++)
				{
					int previousIndex = previousIndexes.get(sortedNames[ndex]).intValue();
					allCodesSortedByName[ndex] = allCodes[previousIndex];
				}
			}
		}
		private static CODES c(){ return insts[Thread.currentThread().getThreadGroup().getName().charAt(0)];}
		public static CODES c(byte c){return insts[c];}
		public static CODES instance(){
			CODES c=insts[Thread.currentThread().getThreadGroup().getName().charAt(0)];
			if(c==null) c=new CODES();
			return c;
		}
		public static void reset() { 
			insts[Thread.currentThread().getThreadGroup().getName().charAt(0)]=null;
			instance();
		}
		private static CODES[] insts=new CODES[256];
		
		private int[] allCodes = new int[0];
		private int[] allCodesSortedByName=new int[0];
		private int[] berries = new int[0];
		private int[] fishes = new int[0];
		private int[][] data =  new int[0][0]; 
		private String[] smells = new String[0];
		private String[] descs = new String[0];
		private String[] effects = new String[0];
		private Ability[][] effectAs = new Ability[0][];
		private SPairList<Integer,Double>[] buckets = null;
		
		/**
		 * Returns an array of the numeric codes for the berry resources
		 * @return an array of the numeric codes for the berry resources
		 */
		public static int[] BERRIES() { return c().berries;}
		
		/**
		 * Returns an array of the numeric codes for the berry resources
		 * @return an array of the numeric codes for the berry resources
		 */
		public int[] berries() { return berries;}
		
		/**
		 * Returns an array of the numeric codes for the fishy resources
		 * @return an array of the numeric codes for the fishy resources
		 */
		public static int[] FISHES() { return c().fishes;}
		
		/**
		 * Returns an array of the numeric codes for the fishy resources
		 * @return an array of the numeric codes for the fishy resources
		 */
		public int[] fishes() { return fishes;}
		/**
		 * Returns total number of codes 0 - this-1
		 * @return total number of codes 0 - this-1
		 */
		public static int TOTAL() { return c().descs.length;}
		/**
		 * Returns total number of codes 0 - this-1
		 * @return total number of codes 0 - this-1
		 */
		public int total() { return descs.length;}
		
		/**
		 * Returns an array of the numeric codes for all resources
		 * @return an array of the numeric codes for all resources
		 */
		public static int[] ALL_SBN() { return c().allCodesSortedByName;}
		/**
		 * Returns an array of the numeric codes for all resources
		 * @return an array of the numeric codes for all resources
		 */
		public static int[] ALL() { return c().allCodes;}
		/**
		 * Returns an array of the numeric codes for all resources
		 * @return an array of the numeric codes for all resources
		 */ 
		public int[] all() { return allCodes;}
		/**
		 * Returns an the numeric codes of the indexes resource code
		 * @param x the indexed resource code
		 * @return an the numeric codes of the indexes resource code
		 */
		public static int GET(int x) { return c().allCodes[x&RESOURCE_MASK];}
		/**
		 * Returns an the numeric codes of the indexes resource code
		 * @param x the indexed resource code
		 * @return an the numeric codes of the indexes resource code
		 */
		public int get(int x) { return allCodes[x&RESOURCE_MASK];}
		/**
		 * Returns the code of the names resource, or -1
		 * @return the code of the names resource, or -1
		 */
		public static int FIND_CaseSensitive(String rsc) {
			if(rsc==null) return -1;
			CODES C=c();
			int x=CMParms.indexOf(C.descs, rsc);
			if(x>=0) return C.allCodes[x];
			return -1;
		}
		/**
		 * Returns the code of the names resource, or -1
		 * @return the code of the names resource, or -1
		 */
		public static int FIND_IgnoreCase(String rsc) {
			if(rsc==null) return -1;
			CODES C=c();
			int x=CMParms.indexOfIgnoreCase(C.descs, rsc);
			if(x>=0) return C.allCodes[x];
			return -1;
		}
		/**
		 * Returns the code of the names resource, or -1
		 * @return the code of the names resource, or -1
		 */
		public static int FIND_StartsWith(String rsc) {
			if(rsc==null) return -1;
			CODES C=c();
			int x=CMParms.startsWith(C.descs, rsc.toUpperCase().trim());
			if(x>=0) return C.allCodes[x];
			return -1;
		}
		/**
		 * Returns whether the code is valid
		 * @return whether the code is valid
		 */
		public static boolean IS_VALID(int code) {
			return (code>=0) && ((code&RawMaterial.RESOURCE_MASK) < c().total());
		}
		/**
		 * Returns the names of the various resources
		 * @return the names of the various resources
		 */
		public static String[] NAMES() { return c().descs;}
		/**
		 * Returns the names of the various resources
		 * @return the names of the various resources
		 */
		public String[] names() { return descs;}
		/**
		 * Returns the name of the code
		 * @param code the code
		 * @return the name of the code
		 */
		public static String NAME(int code) { return c().descs[code&RESOURCE_MASK];}
		/**
		 * Returns the name of the code
		 * @param code the code
		 * @return the name of the code
		 */
		public String name(int code) { return descs[code&RESOURCE_MASK];}
		/**
		 * Returns the smells of the various resources
		 * @return the smells of the various resources
		 */
		public static String[] SMELLS() { return c().smells;}
		/**
		 * Returns the description of the code smell
		 * @param code the code smell
		 * @return the description of the code smell
		 */
		public static String SMELL(int code) { return c().smells[code&RESOURCE_MASK];}
		/**
		 * Returns the description of the code smell
		 * @param code the code smell
		 * @return the description of the code smell
		 */
		public String smell(int code) { return smells[code&RESOURCE_MASK];}
		/**
		 * Returns the smells of the various resources
		 * @return the smells of the various resources
		 */
		public static String[] EFFECTS() { return c().effects;}
		/**
		 * Returns the description of the code smell
		 * @param code the code smell
		 * @return the description of the code smell
		 */
		public static String EFFECT(int code) { return c().effects[code&RESOURCE_MASK];}
		/**
		 * Returns the description of the code smell
		 * @param code the code smell
		 * @return the description of the code smell
		 */
		public String effect(int code) { return effects[code&RESOURCE_MASK];}
		/**
		 * Returns the value of the resource
		 * @return the value of the resource
		 */
		public static int VALUE(int code) { return c().data[code&RESOURCE_MASK][1];}
		/**
		 * Returns the value of the resource
		 * @return the value of the resource
		 */
		public int value(int code) { return data[code&RESOURCE_MASK][1];}
		/**
		 * Returns the frequency of the resource, or how rare it is.
		 * @return the frequency of the resource
		 */
		public static int FREQUENCY(int code) { return c().data[code&RESOURCE_MASK][2];}
		/**
		 * Returns the frequency of the resource, or how rare it is.
		 * @return the frequency of the resource
		 */
		public int frequency(int code) { return data[code&RESOURCE_MASK][2];}
		/**
		 * Returns the hardness of the resource, from 1-10
		 * @return the hardness of the resource
		 */
		public static int HARDNESS(int code) { return c().data[code&RESOURCE_MASK][3];}
		/**
		 * Returns the hardness of the resource, from 1-10
		 * @return the hardness of the resource
		 */
		public int hardness(int code) { return data[code&RESOURCE_MASK][3];}
		/**
		 * Returns the bouancy of the resource, from 0-20000
		 * @return the bouancy of the resource
		 */
		public static int BOUANCY(int code) { return c().data[code&RESOURCE_MASK][4];}
		/**
		 * Returns the bouancy of the resource, from 0-20000
		 * @return the bouancy of the resource
		 */
		public int bouancy(int code) { return data[code&RESOURCE_MASK][4];}
		/**
		 * Search and compose a complete list of all resources of the given material
		 * @param mat the resource code
		 * @return a complete list of all resources of the given material
		 */
		public static List<Integer> COMPOSE_RESOURCES(int mat) {
			if(mat<=RESOURCE_MASK) mat=mat<<8;
			List<Integer> rscs=new Vector<Integer>();
			for(int rsc : c().allCodes)
				if((rsc&MATERIAL_MASK)==mat)
					rscs.add(Integer.valueOf(rsc));
			return rscs;
		}
		
		/**
		 * Parses, if necessary, EFFECT strings into ability objects,
		 * complete with parms, ready for copying.
		 * @param code the material/resource code
		 * @return an ability, if any.
		 */
		@SuppressWarnings("rawtypes")
		public static Ability[] EFFECTA(int code)
		{
			CODES c=c();
			int cd=code&RESOURCE_MASK;
			Ability[] As = c.effectAs[cd];
			if(As!=null) return As;
			synchronized(c.effectAs)
			{
				As = c.effectAs[cd];
				if(As!=null) return As;
				Vector effectsV=CMParms.parseSafeSemicolonList(c.effect(code),true);
				if(effectsV.size()==0)
					c.effectAs[cd]=new Ability[0];
				else
				{
					String abilityID;
					String parms;
					Vector<Ability> listA=new Vector<Ability>();
					for(Enumeration e=effectsV.elements();e.hasMoreElements();)
					{
						abilityID=(String)e.nextElement();
						parms="";
						if((abilityID==null)||(abilityID.length()==0)) continue;
						if(abilityID.charAt(abilityID.length()-1)==')')
						{
							int x=abilityID.indexOf('(');
							if(x>0)
							{
								parms=abilityID.substring(x+1,abilityID.length()-1);
								abilityID=abilityID.substring(0,x);
							}
						}
						Ability A=CMClass.getAbility(abilityID);
						if(A==null)
							Log.errOut("RawMaterial","Unknown ability "+abilityID+" in "+c.effect(code));
						else
						{
							A.setMiscText(parms);
							listA.add(A);
						}
					}
					c.effectAs[cd]=listA.toArray(new Ability[0]);
				}
			}
			return c.effectAs[cd];
		}
		
		@SuppressWarnings("unchecked")
		public SPairList<Integer,Double> getValueSortedBucket(int material)
		{
			material=(material&RawMaterial.MATERIAL_MASK)>>8;
			final int numMaterials=Material.values().length;
			if((material<0)||(material>=numMaterials))
				return null;
			if(buckets == null)
			{
				SPairList<Integer,Double>[] newBuckets=new SPairList[numMaterials];
				for(int matIndex=0;matIndex<numMaterials;matIndex++)
				{
					int matCode = Material.values()[matIndex].mask();
					TreeSet<Pair<Integer,Double>> newBucket=new TreeSet<Pair<Integer,Double>>(
						new Comparator<Pair<Integer,Double>>()
						{
							public int compare(Pair<Integer, Double> o1, Pair<Integer, Double> o2) {
								return o1.second.compareTo(o2.second);
							}
						});
					for(int i=0;i<total();i++)
					{
						int resourceCode=get(i);
						if((resourceCode&RawMaterial.MATERIAL_MASK)==matCode)
						{
							int resourceValue=value( resourceCode );
							newBucket.add(new Pair<Integer,Double>(Integer.valueOf(resourceCode),Double.valueOf(resourceValue)));
						}
					}
					PairSVector<Integer,Double> finalBucket=new PairSVector<Integer,Double>();
					final double pieceSize=1.0 / newBucket.size();
					double currValue = 0.0;
					for(Iterator<Pair<Integer,Double>> i=newBucket.iterator();i.hasNext();)
					{
						Pair<Integer,Double> p=i.next();
						finalBucket.add(p.first, Double.valueOf(currValue));
						currValue += pieceSize;
					}
					newBuckets[matIndex]=finalBucket;
				}
				buckets=newBuckets;
			}
			return buckets[material];
		}
		
		public synchronized void add(int material, String name, String smell, int value, int frequ, int hardness, int bouancy, boolean fish, boolean berry, String abilityID)
		{
			int newResourceCode=allCodes.length | material;
			allCodes=Arrays.copyOf(allCodes, allCodes.length+1);
			allCodes[allCodes.length-1]=newResourceCode;
			if(berry)
			{
				berries=Arrays.copyOf(berries, berries.length+1);
				berries[berries.length-1]=newResourceCode;
			}
			if(fish)
			{
				fishes=Arrays.copyOf(fishes, fishes.length+1);
				fishes[fishes.length-1]=newResourceCode;
			}
			descs=Arrays.copyOf(descs, descs.length+1);
			descs[descs.length-1]=name;
			
			smells=Arrays.copyOf(smells, smells.length+1);
			smells[smells.length-1]=smell;
			
			effects=Arrays.copyOf(effects, effects.length+1);
			effects[effects.length-1]=abilityID;
			effectAs=Arrays.copyOf(effectAs, effectAs.length+1);
			effectAs[effectAs.length-1]=null;
			
			data=Arrays.copyOf(data, data.length+1);
			//full code, base value, frequency, hardness (1-10), bouancy
			int[] newRow={newResourceCode,value,frequ,hardness,bouancy};
			data[data.length-1]=newRow;
		}
		
		public synchronized void replace(int resourceCode, int material, String name, String smell, int value, int frequ, int hardness, int bouancy, boolean fish, boolean berry, String abilityID)
		{
			int resourceIndex = resourceCode & RESOURCE_MASK;
			if((berry)&&(!CMParms.contains(berries, resourceCode)))
			{
				berries=Arrays.copyOf(berries, berries.length+1);
				berries[berries.length-1]=resourceCode;
			}
			else
			if((!berry)&&(CMParms.contains(berries, resourceCode)))
			{
				int[] newberries=new int[berries.length-1];
				int n=0;
				for(int b=0;b<berries.length;b++)
					if(berries[b]!=resourceCode)
						newberries[n++]=berries[b];
				berries=newberries;
			}
			if((fish)&&(!CMParms.contains(fishes, resourceCode)))
			{
				fishes=Arrays.copyOf(fishes, fishes.length+1);
				fishes[fishes.length-1]=resourceCode;
			}
			else
			if((!fish)&&(CMParms.contains(fishes, resourceCode)))
			{
				int[] newfishes=new int[fishes.length-1];
				int n=0;
				for(int b=0;b<fishes.length;b++)
					if(fishes[b]!=resourceCode)
						newfishes[n++]=fishes[b];
				fishes=newfishes;
			}
			smells[resourceIndex]=smell;
			effects[resourceIndex]=abilityID;
			effectAs[resourceIndex]=null;
			descs[resourceIndex]=name;
			int[] newRow={resourceCode,value,frequ,hardness,bouancy};
			data[resourceIndex]=newRow;
		}
	}
}
