package com.planet_ink.coffee_mud.core.collections;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;
/*
Copyright 2000-2013 Bo Zimmerman

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
public class FilteredIterator<K> implements Iterator<K>
{
	private final Iterator<K>  iter;
	private Filterer<K> 	filterer;
	private K 				nextElement = null;
	private boolean 		initialized = false;

	public FilteredIterator(Iterator<K> eset, Filterer<K> fil) 
	{
		iter=eset;
		filterer=fil;
	}
	
	public void setFilterer(Filterer<K> fil) 
	{
		filterer=fil;
	}

	private void stageNextElement()
	{
		nextElement = null;
		while((nextElement==null) && (iter.hasNext()))
		{
			nextElement = iter.next();
			if(filterer.passesFilter(nextElement))
				return;
			nextElement = null;
		}
	}
	
	private void initialize()
	{
		if(!initialized)
		{
			stageNextElement();
			initialized=true;
		}
	}
	
	public boolean hasNext() 
	{ 
		if(!initialized)
			initialize();
		return nextElement!=null;
	}
	
	public K next() 
	{
		if(!hasNext())
			throw new NoSuchElementException();
		K element = nextElement;
		stageNextElement();
		return element;
	}

	public void remove() {
		throw new NoSuchElementException();
	}
}
